package net.astah.stpa.stamp.command;

import java.util.HashSet;
import java.util.Set;
import java.util.function.Supplier;

import org.eclipse.emf.ecore.EObject;

import net.astah.stpa.stamp.Component;
import net.astah.stpa.stamp.ControlStructure;
import net.astah.stpa.stamp.Link;
import net.astah.stpa.stamp.util.STAMPSwitch;

/**
 * STAMP/STPA モデルの複製に関する拡張の実装を提供します。
 */
public class STAMPExtraContentSwitch extends STAMPSwitch<Set<EObject>>
        implements Supplier<Set<EObject>> {

    protected final Set<EObject> contents = new HashSet<>();

    protected final int depth;

    /**
     * インスタンスを構築します。
     * 
     * @param depth
     *            複製処理のネスト段数
     */
    public STAMPExtraContentSwitch(final int depth) {
        this.depth = depth;
    }

    @Override
    public Set<EObject> caseComponent(final Component object) {
        contents.addAll(object.eContents());
        contents.addAll(object.getNestedComponents());
        return super.caseComponent(object);
    }

    @Override
    public Set<EObject> caseControlStructure(final ControlStructure object) {
        contents.addAll(object.eContents());
        contents.addAll(object.getComponents());
        contents.addAll(object.getLinks());
        return super.caseControlStructure(object);
    }

    @Override
    public Set<EObject> caseLink(final Link object) {
        contents.addAll(object.eContents());
        contents.add(object.getSource());
        contents.add(object.getTarget());
        return super.caseLink(object);
    }

    @Override
    public Set<EObject> defaultCase(final EObject object) {
        return get();
    }

    @Override
    public Set<EObject> get() {
        return contents;
    }
}
