package net.astah.stpa.stamp.notation.edit.command;

import static net.astah.stpa.stamp.notation.STAMPNotationPackage.Literals.STAMP_DIAGRAM__OWNER;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.Literals.STAMP_TABLE__OWNER;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;

import net.astah.project.Facet;
import net.astah.project.edit.command.RegisterFacetContentCommand;
import net.astah.stpa.stamp.notation.STAMPDiagram;
import net.astah.stpa.stamp.notation.STAMPNotationPackage;
import net.astah.stpa.stamp.notation.STAMPTable;

/**
 * STAMP の表記法モデルをプロジェクト直下に登録するコマンドです。
 */
public class CreateSTAMPNotationRootCommand extends RegisterFacetContentCommand {

    /**
     * 図表の所有者
     */
    protected final EObject owner;

    /**
     * コンストラクタ
     * 
     * @param domain
     *            コマンドを実行するドメイン
     * @param facet
     *            STPA プロジェクトの {@link Facet}
     * @param owner
     *            図表の所有者
     * @param value
     *            図表
     */
    public CreateSTAMPNotationRootCommand(final EditingDomain domain, final Facet facet,
            final EObject owner, final EObject value) {
        super(domain, facet, STAMPNotationPackage.eCONTENT_TYPE, "model.stampnotation", value);
        this.owner = owner;
    }

    @Override
    protected EObject addFacetContent(final URI resourceURI) {
        final EObject result = super.addFacetContent(resourceURI);
        if (result instanceof STAMPDiagram) {
            append(SetCommand.create(domain, result, STAMP_DIAGRAM__OWNER, owner));
        } else if (result instanceof STAMPTable) {
            append(SetCommand.create(domain, result, STAMP_TABLE__OWNER, owner));
        }
        return result;
    }

}
