package net.astah.stpa.stamp.notation.edit.command;

import static net.astah.stpa.stamp.notation.STAMPNotationPackage.Literals.STAMP_DIAGRAM_ELEMENT__STAMP_MODEL_ELEMENT;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.Literals.STAMP_LABEL__TEXT;

import java.util.Objects;

import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;

import com.google.common.base.Strings;

import net.astah.stpa.stamp.Element;
import net.astah.stpa.stamp.NamedElement;
import net.astah.stpa.stamp.Note;
import net.astah.stpa.stamp.notation.STAMPLabel;

/**
 * ラベル図要素の表記を所有モデルと同期するコマンドです。
 */
public class SynchronizeLabelCommand extends CompoundCommand {

    private EditingDomain domain;

    private STAMPLabel owner;

    /**
     * コンストラクタ
     * 
     * @param domain
     *            コマンドを実行するドメイン
     * @param owner
     *            同期対象となるラベル図要素
     */
    public SynchronizeLabelCommand(final EditingDomain domain, final STAMPLabel owner) {
        this.domain = domain;
        this.owner = owner;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected boolean prepare() {
        prepareCommand();
        return super.prepare();
    }

    private void prepareCommand() {
        if ((domain == null) || (owner == null)) {
            return;
        }
        final String text = getText();
        if (Strings.isNullOrEmpty(text)) {
            append(SetCommand.create(domain, owner, STAMP_LABEL__TEXT, SetCommand.UNSET_VALUE));
        } else {
            if (!Objects.equals(text, owner.getText())) {
                append(SetCommand.create(domain, owner, STAMP_LABEL__TEXT, text));
            }
        }
    }

    private String getText() {
        if (owner.eIsSet(STAMP_DIAGRAM_ELEMENT__STAMP_MODEL_ELEMENT)) {
            for (final Element element : owner.getSTAMPModelElements()) {
                if (element instanceof NamedElement) {
                    return ((NamedElement) element).getName();
                } else if (element instanceof Note) {
                    return ((Note) element).getBody();
                }
            }
        }
        return null;
    }

}
