package net.astah.stpa.stamp.notation.edit.reactor;

import static net.astah.stpa.stamp.STAMPPackage.Literals.UCA_ANALYSIS__CONTROL_ACTION;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.Literals.STAMP_CONTROL_STRUCTURE_DIAGRAM;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.Literals.UCA_TABLE;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.Literals.UCA_TABLE__ANALYSIS;
import static net.astah.stpa.stamp.notation.STAMPNotationPackage.Literals.UCA_TABLE__CONTROL_STRUCTURE_DIAGRAM;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.domain.EditingDomain;

import net.astah.common.collect.Streams;
import net.astah.dd.di.util.DiagramElements;
import net.astah.emf.common.ecore.EObjects;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.common.notify.IPathNotification;
import net.astah.emf.edit.notify.EFeaturePathReactor;
import net.astah.stpa.stamp.ControlAction;
import net.astah.stpa.stamp.ControlLink;
import net.astah.stpa.stamp.UCAAnalysis;
import net.astah.stpa.stamp.notation.STAMPEdge;

/**
 * コントロールストラクチャ図でコントロールアクションが追加された時に UCA 表への同期処理を行うクラスです。
 *
 */
public class ControlLinkSynchronizer implements EFeaturePathReactor {

    /**
     * @see EFeaturePathReactor#process(EditingDomain, Collection)
     */
    @Override
    public Command process(final EditingDomain domain,
            final Collection<IPathNotification> notifications) {
        final Set<Object> processed = new HashSet<>();
        final CompoundCommand result = new CompoundCommand();

        for (final IPathNotification notification : notifications) {
            final Object notifier = notification.getNotifier();
            if (processed.add(notifier)) {
                final STAMPEdge edge = (STAMPEdge) notification.getNotifier();
                final EObject element = DiagramElements.getModelElement(edge);
                final Optional<UCAAnalysis> ucaAnalysis = getUCAAnalysis(edge);
                if (ucaAnalysis.isPresent()) {
                    final UCAAnalysis analysis = ucaAnalysis.get();
                    if (element instanceof ControlLink) {
                        // UCAAnalysis#getActionsに含まれないControlActionを追加する
                        final List<ControlAction> actions = ((ControlLink) element).getActions()
                                .stream()
                                .filter(action -> !analysis.getControlActions().contains(action))
                                .collect(Collectors.toList());
                        for (final ControlAction action : actions) {
                            result.appendIfCanExecute(AddCommand.create(domain, analysis,
                                    UCA_ANALYSIS__CONTROL_ACTION, action));
                        }
                    }
                }
            }
        }
        return result.isEmpty() ? null : result.unwrap();
    }

    private Optional<UCAAnalysis> getUCAAnalysis(final STAMPEdge edge) {
        final EObject diagram = EObjects.getAncestor(STAMP_CONTROL_STRUCTURE_DIAGRAM, edge);
        final EFeaturePath featurePath = EFeaturePath.builder()
                .addInverse(STAMP_CONTROL_STRUCTURE_DIAGRAM, UCA_TABLE__CONTROL_STRUCTURE_DIAGRAM)
                .add(UCA_TABLE, UCA_TABLE__ANALYSIS).create();
        return Streams.filterByType(featurePath.getReferences(diagram).stream(), UCAAnalysis.class)
                .findFirst();
    }
}
