package net.astah.stpa.stamp.notation.editor;

import java.util.List;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.DragAndDropCommand;
import org.eclipse.emf.edit.command.DragAndDropFeedback;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.dd.di.DiagramElement;
import net.astah.dd.di.util.DiagramElements;
import net.astah.notation.Shape;
import net.astah.stpa.stamp.ProcessValue;
import net.astah.stpa.stamp.ProcessVariable;

/**
 * プロセスモデル要素関連のユーティリティクラスです。
 */
public class ProcessModelElements {
    /**
     * 移動(順序変更)を実行するコマンドを生成します。
     * 
     * @param selectedModels
     *            選択されたモデル
     * @param destinationLabel
     *            移動先のラベル
     * @param y
     *            移動先のy座標
     * @return 移動を実行するコマンド
     */
    public static Command createCommandToReorder(final List<EObject> selectedModels,
            final Shape destinationLabel, final double y) {
        final EditingDomain domain = TransactionUtil.getEditingDomain(selectedModels.get(0));
        final EObject destinationModel = DiagramElements.getModelElement(destinationLabel);
        final float location = getLocation(destinationLabel, y);
        return DragAndDropCommand.create(domain, destinationModel, location,
                DragAndDropFeedback.DROP_MOVE, DragAndDropFeedback.DROP_MOVE, selectedModels);
    }

    /**
     * 移動先のラベル上の比率での位置(0～1)を計算します。
     * 
     * @param destinationLabel
     *            移動先のラベル
     * @param y
     *            移動先のy座標
     * @return 移動先のラベル上の比率での位置(0～1)
     */
    public static float getLocation(final Shape destinationLabel, final double y) {
        return (float) ((y - destinationLabel.getBounds().getY())
                / destinationLabel.getBounds().getHeight());
    }

    static boolean isProcessModelElement(final DiagramElement diagramElement) {
        final EObject model = DiagramElements.getModelElement(diagramElement);
        return model instanceof ProcessVariable || model instanceof ProcessValue;
    }
}
