package net.astah.stpa.stamp.notation.editor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.stream.Collectors;

import net.astah.dd.di.DiagramElement;
import net.astah.dd.di.Edge;
import net.astah.jomt.jview.BaseHighlighter;
import net.astah.jomt.jview.Highlighter;
import net.astah.notation.editor.ViewElements;
import net.astah.stpa.stamp.NoteAnchor;
import net.astah.stpa.stamp.notation.STAMPComponentShape;

/**
 * コンポーネント要素のハイライト状態を制御するためのクラスです。
 */
public class STAMPComponentHighlighter extends BaseHighlighter {

    public STAMPComponentHighlighter(STAMPComponentViewElement target) {
        super(target);
    }

    @Override
    public Collection<Highlighter> getHighlightTogether() {
        Collection<Highlighter> results = new HashSet<>();
        List<DiagramElement> items = getHighlightDiagramElements(getTarget().getModel());
        items.forEach(de -> ViewElements.viewElementsOf(de)
                .forEach(ve -> results.add(ve.getAdapter(Highlighter.class))));
        return results;
    }

    private List<DiagramElement> getHighlightDiagramElements(Object object) {
        List<DiagramElement> items = new ArrayList<>();
        if (object instanceof STAMPComponentShape) {
            final STAMPComponentShape shape = (STAMPComponentShape) object;
            items.addAll(shape.getSourceEdges().stream().filter(edge -> !isNoteAnchor(edge))
                    .collect(Collectors.toList()));
            items.addAll(shape.getTargetEdges().stream().filter(edge -> !isNoteAnchor(edge))
                    .collect(Collectors.toList()));
            items.add(shape);
        }
        return items;
    }

    private boolean isNoteAnchor(Edge edge) {
        return edge.getModelElements().stream().anyMatch(model -> model instanceof NoteAnchor);
    }
}
