package net.astah.stpa.stamp.ui;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.transaction.util.TransactionUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.collect.Lists;

import net.astah.golf.control.Mode;
import net.astah.golf.draw.GNode;
import net.astah.jomt.jcontrol.mode.DiagramZoomMode;
import net.astah.jomt.jcontrol.mode.MoveFeatureMode;
import net.astah.notation.editor.IItemEditorFactory;
import net.astah.notation.editor.ItemEditorFactoryImpl;
import net.astah.notation.editor.graphics.GraphicalShapeViewElementFactory;
import net.astah.stpa.stamp.ControlLink;
import net.astah.stpa.stamp.Element;
import net.astah.stpa.stamp.FeedbackLink;
import net.astah.stpa.stamp.LinkEnd;
import net.astah.stpa.stamp.Note;
import net.astah.stpa.stamp.NoteAnchor;
import net.astah.stpa.stamp.notation.AccidentHazardSafetyConstraintTable;
import net.astah.stpa.stamp.notation.ComponentExtractingTable;
import net.astah.stpa.stamp.notation.CountermeasureTable;
import net.astah.stpa.stamp.notation.HCFTable;
import net.astah.stpa.stamp.notation.PreconditionTable;
import net.astah.stpa.stamp.notation.STAMPCompartment;
import net.astah.stpa.stamp.notation.STAMPComponentShape;
import net.astah.stpa.stamp.notation.STAMPControlLoopDiagram;
import net.astah.stpa.stamp.notation.STAMPControlStructureDiagram;
import net.astah.stpa.stamp.notation.STAMPDiagram;
import net.astah.stpa.stamp.notation.STAMPEdge;
import net.astah.stpa.stamp.notation.STAMPGraphicalShape;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.stpa.stamp.notation.STAMPNotationPackage;
import net.astah.stpa.stamp.notation.STAMPShape;
import net.astah.stpa.stamp.notation.UCATable;
import net.astah.stpa.stamp.notation.editor.STAMPCompartmentViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPComponentViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPControlLinkViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPControlLoopDiagramViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPControlStructureDiagramViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPFeedbackLinkViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPLabelViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPLinkEndViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPNoteAnchorViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPNoteViewElement;
import net.astah.stpa.stamp.notation.editor.STAMPOmittedComponentViewElement;
import net.astah.stpa.stamp.notation.util.STAMPNotationAdapterFactory;
import net.astah.stpa.stamp.notation.util.STAMPNotationSwitch;
import net.astah.stpa.stamp.ui.control.mode.ControlStructureDiagramDnDMode;
import net.astah.stpa.stamp.ui.view.swing.AccidentHazardSafetyConstraintTableEditorPeer;
import net.astah.stpa.stamp.ui.view.swing.ComponentExtractingTableEditorPeer;
import net.astah.stpa.stamp.ui.view.swing.CountermeasureTableEditorPeer;
import net.astah.stpa.stamp.ui.view.swing.HCFTableEditorPeer;
import net.astah.stpa.stamp.ui.view.swing.PreconditionTableEditorPeer;
import net.astah.stpa.stamp.ui.view.swing.STAMPDiagramEditorPeer;
import net.astah.stpa.stamp.ui.view.swing.UCATableEditorPeer;
import net.astah.ui.control.mode.DiagramPopupMode;
import net.astah.ui.control.mode.MoveMode;
import net.astah.ui.control.mode.RelationMode;
import net.astah.ui.control.mode.ResizeMode;

/**
 * Adapter の作成や図の初期処理を行うクラスです。
 */
public class STAMPNotationEditorAdapterFactory extends STAMPNotationAdapterFactory {

    private static final Logger LOG = LoggerFactory
            .getLogger(STAMPNotationEditorAdapterFactory.class);

    protected Collection<Object> supportedTypes = new HashSet<>();

    protected Adapter adapter;

    public STAMPNotationEditorAdapterFactory() {
        supportedTypes.add(IItemEditorFactory.class);
    }

    @Override
    public Adapter createAdapter(Notifier target) {
        if (adapter == null) {
            adapter = new ItemEditorFactoryImpl(new CreateSwitch(), new CreateEditorSwitch(),
                    new AssociationModeSwitch(), new LineSwitch(), new ModeToolBarSwitch(),
                    STAMPNotationPackage.eINSTANCE.getSTAMPGraphicalShape());
        }
        return adapter;
    }

    @Override
    public boolean isFactoryForType(final Object type) {
        return supportedTypes.contains(type) || super.isFactoryForType(type);
    }

    /**
     * モデルごとに ViewElement を作成します。
     */
    protected static class CreateSwitch extends STAMPNotationSwitch<GNode> {

        @Override
        public GNode caseSTAMPControlStructureDiagram(STAMPControlStructureDiagram object) {
            final STAMPControlStructureDiagramViewElement result = new STAMPControlStructureDiagramViewElement(
                    object);
            result.setDomain(TransactionUtil.getEditingDomain(object));
            result.refresh(null, null);
            return result;
        }

        @Override
        public GNode caseSTAMPControlLoopDiagram(STAMPControlLoopDiagram object) {
            final STAMPControlLoopDiagramViewElement result = new STAMPControlLoopDiagramViewElement(
                    object);
            result.setDomain(TransactionUtil.getEditingDomain(object));
            result.refresh(null, null);
            return result;
        }

        @Override
        public GNode caseSTAMPComponentShape(STAMPComponentShape object) {
            if (object.isOmitted()) {
                return new STAMPOmittedComponentViewElement(object);
            }
            return new STAMPComponentViewElement(object);
        }

        @Override
        public GNode caseSTAMPCompartment(STAMPCompartment object) {
            return new STAMPCompartmentViewElement(object);
        }

        @Override
        public GNode caseSTAMPEdge(final STAMPEdge object) {
            for (Element element : object.getSTAMPModelElements()) {
                if (element instanceof ControlLink) {
                    return new STAMPControlLinkViewElement(object);
                } else if (element instanceof FeedbackLink) {
                    return new STAMPFeedbackLinkViewElement(object);
                } else if (element instanceof NoteAnchor) {
                    return new STAMPNoteAnchorViewElement(object);
                }
            }
            return null;
        }

        @Override
        public GNode caseSTAMPLabel(STAMPLabel object) {
            return new STAMPLabelViewElement(object);
        }

        @Override
        public GNode caseSTAMPShape(final STAMPShape object) {
            for (Element element : object.getSTAMPModelElements()) {
                if (element instanceof Note) {
                    return new STAMPNoteViewElement(object);
                }

                if (element instanceof LinkEnd) {
                    return new STAMPLinkEndViewElement(object);
                }
            }
            return null;
        }

        @Override
        public GNode caseSTAMPGraphicalShape(STAMPGraphicalShape object) {
            return GraphicalShapeViewElementFactory.INSTANCE.createViewElement(object);
        }
    }

    /**
     * 図表ごとに EditorPeer を作成します。
     */
    protected static class CreateEditorSwitch extends STAMPNotationSwitch<Object> {

        @Override
        public Object caseSTAMPControlStructureDiagram(STAMPControlStructureDiagram object) {
            return new STAMPDiagramEditorPeer();
        }

        @Override
        public Object caseSTAMPControlLoopDiagram(STAMPControlLoopDiagram object) {
            return new STAMPDiagramEditorPeer();
        }

        @Override
        public Object casePreconditionTable(PreconditionTable object) {
            return new PreconditionTableEditorPeer();
        }

        @Override
        public Object caseAccidentHazardSafetyConstraintTable(
                AccidentHazardSafetyConstraintTable object) {
            return new AccidentHazardSafetyConstraintTableEditorPeer();
        }

        @Override
        public Object caseUCATable(UCATable object) {
            return new UCATableEditorPeer();
        }

        @Override
        public Object caseHCFTable(HCFTable object) {
            return new HCFTableEditorPeer();
        }

        @Override
        public Object caseCountermeasureTable(CountermeasureTable object) {
            return new CountermeasureTableEditorPeer();
        }

        @Override
        public Object caseComponentExtractingTable(ComponentExtractingTable object) {
            return new ComponentExtractingTableEditorPeer();
        }

    }

    /**
     * 図ごとに実行できる Mode を設定します。
     */
    protected static class AssociationModeSwitch extends STAMPNotationSwitch<List<Mode>> {
        @Override
        public List<Mode> caseSTAMPDiagram(STAMPDiagram object) {
            List<Mode> modes = Lists.newArrayList();
            try {
                // common mode
                modes.add(new MoveMode());
                modes.add(new DiagramPopupMode());
                modes.add(new DiagramZoomMode());
                modes.add(new ResizeMode());
                modes.add(new MoveFeatureMode());
                modes.add(new RelationMode());
                // STAMP mode
                modes.add(new ControlStructureDiagramDnDMode());

            } catch (Exception e) {
                LOG.warn("Failed to get mode for " + object.getName(), e);
            }
            return modes;
        }
    }

    /**
     * 図ごとにツールバーを取得するキーを返します。
     */
    protected static class ModeToolBarSwitch extends STAMPNotationSwitch<String> {
        @Override
        public String caseSTAMPControlStructureDiagram(STAMPControlStructureDiagram object) {
            return "diagramview.modetoolbar.controlstructuredgm_modetoolbar";
        }

        @Override
        public String caseSTAMPControlLoopDiagram(STAMPControlLoopDiagram object) {
            return "diagramview.modetoolbar.controlloopdgm_modetoolbar";
        }
    }

    /**
     * 図ごとにデフォルトの線種を取得するキーを返します。
     */
    protected static class LineSwitch extends STAMPNotationSwitch<String> {
        @Override
        public String caseSTAMPControlStructureDiagram(STAMPControlStructureDiagram object) {
            return "stpa.controlstructure_diagram.line_shape.selected";
        }

        @Override
        public String caseSTAMPControlLoopDiagram(STAMPControlLoopDiagram object) {
            return "stpa.controlloop_diagram.line_shape.selected";
        }
    }

}
