package net.astah.stpa.stamp.ui.control;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.domain.EditingDomain;

import net.astah.dd.di.Diagram;
import net.astah.emf.edit.command.Commands;
import net.astah.golf.control.Command;
import net.astah.golf.model.IEntityStore;
import net.astah.jomt.jcontrol.OpenTableEditorCommand;
import net.astah.jomt.jsystem.PropertyManager;
import net.astah.project.Facet;
import net.astah.project.Facets;
import net.astah.project.Project;
import net.astah.project.ProjectManagerAdapter;
import net.astah.stpa.stamp.STAMPPlugin;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.notation.AccidentHazardSafetyConstraintTable;
import net.astah.stpa.stamp.notation.STAMPNotationFactory;
import net.astah.stpa.stamp.notation.edit.command.CreateSTAMPNotationRootCommand;
import net.astah.uml.exception.IllegalModelTypeException;

/**
 * アクシデントハザード安全制約表を作成するコマンドです。
 */
public class CreateAccidentHazardSafetyConstraintTableFromPrjCommand
        extends CreateSTAMPDiagramFromPrjCommand {

    @Override
    protected Diagram findSingletonDiagram(final EObject selectedModel) {
        final Project project = ProjectManagerAdapter.getRootProject(selectedModel);
        return Facets.getFirstContent(project, STAMPPlugin.FACET_SYMBOLIC_NAME,
                AccidentHazardSafetyConstraintTable.class, null);
    }

    @Override
    protected Diagram createNewDiagram(IEntityStore doc, EObject owner) {
        final EditingDomain domain = doc.getDomain();
        final Project project = ProjectManagerAdapter.getRootProject(owner);
        final Facet facet = project.getFacet(STAMPPlugin.FACET_SYMBOLIC_NAME);
        final STPAAnalysis analysis = (STPAAnalysis) facet.getContents().get(0);

        AccidentHazardSafetyConstraintTable table = STAMPNotationFactory.eINSTANCE
                .createAccidentHazardSafetyConstraintTable();
        Commands.execute(new CreateSTAMPNotationRootCommand(domain, facet, owner, table));
        table.setScope(analysis);

        return table;
    }

    @Override
    protected void setDiagramName(IEntityStore doc, Diagram diagram) {
        diagram.setName(PropertyManager.getProjectViewResourceBundle()
                .getString("uml.accident_hazard_safety_constraint_table.label"));
    }

    @Override
    protected String getNameString() {
        throw new UnsupportedOperationException();
    }

    @Override
    protected void doPostProcessing() throws IllegalModelTypeException {
        // 親の処理を無効化
    }

    @Override
    protected Command createOpenEditorCommand(Diagram diagram) {
        OpenTableEditorCommand command = new OpenTableEditorCommand();
        command.setModel(diagram);
        return command;
    }
}
