package net.astah.stpa.stamp.ui.control;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.job.Jobs;
import net.astah.golf.draw.GNode;
import net.astah.golf.widget.CCanvas;
import net.astah.jomt.jsystem.JomtGlobal;
import net.astah.jomt.jview.DiagramEditor;
import net.astah.jomt.jview.DrawSuggest;
import net.astah.notation.editor.IViewElement;
import net.astah.notation.editor.ViewElements;
import net.astah.stpa.stamp.notation.STAMPCompartment;
import net.astah.stpa.stamp.notation.STAMPCompartmentableShape;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.stpa.stamp.notation.STAMPNotationFactory;
import net.astah.stpa.stamp.notation.STAMPNotationPackage;

/**
 * コンパートメントを作成するコマンドです。
 */
public class CreateCompartmentCommand extends net.astah.golf.control.Command {

    private STAMPCompartmentableShape shape;

    public CreateCompartmentCommand() {
    }

    public void setShape(STAMPCompartmentableShape shape) {
        this.shape = shape;
    }

    @Override
    public void execute() {
        if (shape == null) {
            shape = getSelectedCompartmentableShape();
            if (shape == null) {
                return;
            }
        }

        TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(shape);

        // このコマンドが実行するEMFコマンドを構築する
        Command command = createTransactionalCommand(domain);

        // 構築したEMFコマンドを非同期ジョブとして実行する
        CommandJob job = new CommandJob(domain, command);
        Jobs.toCompletableFuture(job, true).thenRun(this::postExecute);
        job.schedule();
    }

    /**
     * トランザクションとして実行するモデル編集コマンドの作成.
     * 
     * @param domain
     *            コマンドを実行するドメイン
     * @return 作成したコマンド
     */
    public Command createTransactionalCommand(TransactionalEditingDomain domain) {
        STAMPCompartment compartment = STAMPNotationFactory.eINSTANCE.createSTAMPCompartment();
        STAMPLabel label = (STAMPLabel) compartment
                .createSTAMPOwnedElement(STAMPNotationPackage.Literals.STAMP_LABEL);
        label.setText(String.format("Compartment%d", shape.getCompartments().size()));
        return AddCommand.create(domain, shape,
                STAMPNotationPackage.Literals.STAMP_COMPARTMENTABLE_SHAPE__COMPARTMENT,
                compartment);
    }

    private STAMPCompartmentableShape getSelectedCompartmentableShape() {
        final Object source = event.getSource();
        if (source instanceof DrawSuggest) {
            final Notifier model = ((DrawSuggest) source).getOwner().getModel();
            if (model instanceof STAMPCompartmentableShape) {
                return (STAMPCompartmentableShape) model;
            }
        }
        return null;
    }

    /**
     * トランザクション完了後の後処理.
     */
    protected void postExecute() {
        DiagramEditor editor = JomtGlobal.viewManager.getCurrentDiagramEditor();
        CCanvas canvas = editor.getCanvas();
        canvas.unselect();

        for (IViewElement iViewElement : ViewElements.viewElementsOf(shape)) {
            canvas.selectAndNotify(((GNode) iViewElement).getChildren());
            canvas.repaint();
        }
    }

}
