package net.astah.stpa.stamp.ui.control;

import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.edit.domain.EditingDomain;

import net.astah.golf.control.Command;
import net.astah.jomt.jsystem.ModelGlobal;
import net.astah.project.Facets;
import net.astah.project.Project;
import net.astah.project.ProjectManagerAdapter;
import net.astah.stpa.stamp.STAMPPlugin;
import net.astah.stpa.stamp.notation.ComponentExtractingTable;
import net.astah.stpa.stamp.notation.STAMPControlStructureDiagram;

/**
 * コントロールストラクチャー図の作成を他のコマンドに移譲します。
 * <p>
 * 主に、メニューからのアクションイベントによる呼び出しを想定していて、移譲するコマンドを決定して実行します。
 * </p>
 */
public class CreateControlStructureDiagramDelegateCommand extends Command {

    private String arg;
    private Project project;

    @Override
    public void setArgumentString(final String arg) {
        this.arg = arg;
    }

    @Override
    public void execute() {
        project = getRootProject();
        callCommand(createCommand());
    }

    private Project getRootProject() {
        final net.astah.jmodel.Project currentProject = ModelGlobal.prjManager.getCurrentProject();
        if (currentProject == null) {
            return null;
        }
        final EditingDomain domain = currentProject.doc.getDomain();
        if (domain == null) {
            return null;
        }
        final ResourceSet resourceSet = domain.getResourceSet();
        if (resourceSet == null) {
            return null;
        }
        final Project rootProject = ProjectManagerAdapter.getRootProject(resourceSet);
        return rootProject;
    }

    private Command createCommand() {
        final ComponentExtractingTable extractingTable = getComponentExtractingTable();

        final Command command;
        if (!existControlStructureDiagram() && extractingTable != null) {
            final CreateControlStructureDiagramFromTableCommand fromTableCommand = new CreateControlStructureDiagramFromTableCommand();
            fromTableCommand.setComponentExtractingTable(extractingTable);
            command = fromTableCommand;
        } else {
            command = new CreateControlStructureDiagramFromPrjCommand();
        }

        if (arg != null) {
            command.setArgumentString(arg);
        }
        return command;
    }

    private boolean existControlStructureDiagram() {
        return Facets.getContents(project, STAMPPlugin.FACET_SYMBOLIC_NAME).stream()
                .anyMatch(STAMPControlStructureDiagram.class::isInstance);
    }

    private ComponentExtractingTable getComponentExtractingTable() {
        return Facets.getFirstContent(project, STAMPPlugin.FACET_SYMBOLIC_NAME,
                ComponentExtractingTable.class, null);
    }

}
