package net.astah.stpa.stamp.ui.control;

import static net.astah.stpa.stamp.notation.STAMPNotationPackage.Literals.HCF_TABLE__UNSAFE_CONTROL_ACTION;

import java.awt.Component;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.edit.domain.EditingDomain;

import com.google.common.base.Strings;

import net.astah.dd.di.Diagram;
import net.astah.emf.common.ecore.EComparators;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.command.Commands;
import net.astah.emf.edit.provider.swing.renderer.LabelProviderListCellRenderer;
import net.astah.golf.control.Command;
import net.astah.golf.model.IEntityStore;
import net.astah.golf.widget.swing.ILabelProvider;
import net.astah.golf.widget.swing.SwingFrame;
import net.astah.jomt.jcontrol.OpenTableEditorCommand;
import net.astah.jomt.jsystem.JomtGlobal;
import net.astah.jomt.jsystem.PropertyManager;
import net.astah.jomt.jview.MessageManager;
import net.astah.project.Facet;
import net.astah.project.Facets;
import net.astah.project.Project;
import net.astah.project.ProjectManagerAdapter;
import net.astah.stpa.stamp.ControlAction;
import net.astah.stpa.stamp.ControlLoop;
import net.astah.stpa.stamp.HCFAnalysis;
import net.astah.stpa.stamp.HCFHintWordSet;
import net.astah.stpa.stamp.STAMPPlugin;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.UnsafeControlAction;
import net.astah.stpa.stamp.notation.HCFTable;
import net.astah.stpa.stamp.notation.STAMPNotationFactory;
import net.astah.stpa.stamp.notation.edit.command.CreateSTAMPNotationRootCommand;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.view.SelectElementListDialog;
import net.astah.uml.exception.IllegalModelTypeException;

/**
 * HCF 表を作成するコマンドです。
 */
public class CreateHCFTableFromPrjCommand extends CreateSTAMPDiagramFromPrjCommand {

    @Override
    protected Diagram createNewDiagram(final IEntityStore doc, final EObject owner) {
        final EditingDomain domain = doc.getDomain();
        final Project project = ProjectManagerAdapter.getRootProject(owner);
        final Facet facet = project.getFacet(STAMPPlugin.FACET_SYMBOLIC_NAME);
        final STPAAnalysis analysis = (STPAAnalysis) facet.getContents().get(0);

        final JFrame parent = (JFrame) ((SwingFrame) JomtGlobal.viewManager.getManagementView()
                .getComponent()).getComponent();
        final List<UnsafeControlAction> list = new ArrayList<>(analysis.getUnsafeControlActions());
        if (list.isEmpty()) {
            MessageManager.displayConfirmMessageDialog("",
                    Messages.create_hcf_table_confirm_no_uca);
            return null;
        }
        list.sort(EComparators.comparingFeature("id"));

        final UnsafeControlAction initialSelected = list.get(0);
        final SelectElementListDialog<UnsafeControlAction> dialog = new SelectElementListDialog<UnsafeControlAction>(
                parent, list, initialSelected) {
            private static final long serialVersionUID = 1L;

            @Override
            protected ListCellRenderer<? super UnsafeControlAction> createListCellRenderer() {
                return new UCAListCellRenderer(createLabelProvider());
            }
        };

        dialog.setTitle(Messages.hcf_table_select_uca_label);
        dialog.setDescription(Messages.hcf_table_select_uca_description_label);
        dialog.setVisible(true);

        if (dialog.getResultState() != SelectElementListDialog.OK) {
            dialog.dispose();
            return null;
        }

        final UnsafeControlAction uca = dialog.getSelectedElement();
        dialog.dispose();

        final Set<EObject> hcfTables = EFeaturePath.build(HCF_TABLE__UNSAFE_CONTROL_ACTION)
                .getInverseReferences(uca);
        if (!hcfTables.isEmpty()) {
            return (HCFTable) hcfTables.iterator().next();
        }

        HCFAnalysis hcfAnalysis = analysis.getHcfAnalysis();
        if (hcfAnalysis == null) {
            // HCFAnalysisを作成する
            hcfAnalysis = analysis.createHcfAnalysis();
        }

        final ControlAction controlAction = uca.getControlAction();
        ControlLoop controlLoop = getControlLoop(hcfAnalysis, controlAction);
        if (controlLoop == null) {
            // ControlLoopを作成する
            controlLoop = hcfAnalysis.createControlLoop();
            controlLoop.setTargetLink(controlAction.getOwner());

            // デフォルトヒントワードセットを設定する
            final HCFHintWordSet defaultHintWordSet = analysis.getHCFHintWordSets().get(0);
            controlLoop.setHintWordSet(defaultHintWordSet);
        }

        final HCFTable table = STAMPNotationFactory.eINSTANCE.createHCFTable();
        Commands.execute(new CreateSTAMPNotationRootCommand(domain, facet, owner, table));
        table.setControlLoop(controlLoop);
        table.setUnsafeControlAction(uca);

        return table;
    }

    private ControlLoop getControlLoop(final HCFAnalysis hcfAnalysis,
            final ControlAction controlAction) {
        final EList<ControlLoop> controlLoops = hcfAnalysis.getControlLoops();
        return controlLoops.stream().filter(loop -> loop.getTargetLink() == controlAction.getLink())
                .findFirst().orElse(null);
    }

    @Override
    protected void setDiagramName(final IEntityStore doc, final Diagram diagram) {
        final ResourceSet resourceSet = doc.getDomain().getResourceSet();
        if (!Strings.isNullOrEmpty(diagram.getName())) {
            return;
        }
        final String prefix = PropertyManager.getProjectViewResourceBundle()
                .getString("uml.hcf_table.label");
        diagram.setName(Facets.generateName(resourceSet, STAMPPlugin.FACET_SYMBOLIC_NAME,
                diagram.eClass(), prefix));
    }

    @Override
    protected String getNameString() {
        throw new UnsupportedOperationException();
    }

    @Override
    protected void doPostProcessing() throws IllegalModelTypeException {
        // 親の処理を無効化
    }

    @Override
    protected Command createOpenEditorCommand(final Diagram diagram) {
        final OpenTableEditorCommand command = new OpenTableEditorCommand();
        command.setModel(diagram);
        return command;
    }

    /**
     * UCAに対応するHCF表の有無も合わせて表示するための ListCellRenderer です。
     */
    private static class UCAListCellRenderer extends LabelProviderListCellRenderer {
        private static final long serialVersionUID = 1L;

        private final EFeaturePath featurePath = EFeaturePath
                .build(HCF_TABLE__UNSAFE_CONTROL_ACTION);

        public UCAListCellRenderer(final ILabelProvider labelProvider) {
            super(labelProvider);
        }

        @Override
        public Component getListCellRendererComponent(final JList<?> list, final Object value,
                final int index, final boolean isSelected, final boolean cellHasFocus) {
            final Component component = super.getListCellRendererComponent(list, value, index,
                    isSelected, cellHasFocus);
            assert component instanceof JLabel;
            final JLabel label = (JLabel) component;
            final JLabel checkLabel = new JLabel("○ ");
            checkLabel.setFont(label.getFont());

            final StringBuffer text = new StringBuffer();
            if (featurePath.getInverseReferences((EObject) value).isEmpty()) {
                label.setBorder(new CompoundBorder(label.getBorder(),
                        new EmptyBorder(0, checkLabel.getPreferredSize().width, 0, 0)));
            } else {
                text.append(checkLabel.getText());
            }
            text.append(label.getText());
            label.setText(text.toString());

            return label;
        }
    };

}
