package net.astah.stpa.stamp.ui.control.mode;

import java.awt.event.MouseEvent;
import java.util.Collections;

import org.eclipse.emf.ecore.EModelElement;

import net.astah.dd.di.Diagram;
import net.astah.dd.di.DiagramElement;
import net.astah.golf.control.CommandEvent;
import net.astah.golf.draw.GNode;
import net.astah.notation.Edge;
import net.astah.notation.GraphicalShape;
import net.astah.notation.LineConstraint;
import net.astah.notation.NotationFactory;
import net.astah.notation.Shape;
import net.astah.notation.editor.ViewElements;
import net.astah.stpa.stamp.Note;
import net.astah.stpa.stamp.NoteAnchor;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.notation.STAMPNotationFactory;
import net.astah.stpa.stamp.notation.STAMPNotationPackage;
import net.astah.ui.control.CreateEdgeCommand;
import net.astah.ui.control.mode.CreateRelationMode;

/**
 * ノートアンカーを作成する Mode です。
 */
public class CreateNoteAnchorMode extends CreateRelationMode {

    @Override
    protected Edge createEdge() {
        return STAMPNotationFactory.eINSTANCE.createSTAMPEdge();
    }

    @Override
    protected EModelElement createModelElement() {
        return STAMPFactory.eINSTANCE.createNoteAnchor();
    }

    @Override
    public void fireCommand(final MouseEvent e) {
        DiagramElement sourceDiagramElement = getDiagramElement(source);
        DiagramElement targetDiagramElement = getDiagramElement(target);
        if (sourceDiagramElement == targetDiagramElement) {
            return;
        }

        boolean sourceHasNote = hasModel(sourceDiagramElement, Note.class);
        boolean targetHasNote = hasModel(targetDiagramElement, Note.class);
        if (!sourceHasNote && !targetHasNote) {
            return;
        }

        // source側が必ずNoteになるようにする
        if (!sourceHasNote) {
            final GNode temp = source;
            source = target;
            target = temp;
            sourceDiagramElement = getDiagramElement(source);
            targetDiagramElement = getDiagramElement(target);
            Collections.reverse(points);
        }

        final CreateEdgeCommand command = getCommand();
        final Object[] edgeFactor = createEdgeFactor();
        command.setEdge((Edge) edgeFactor[0]);
        command.setSourceAnchor(edgeFactor[1]);
        command.setTargetAnchor(edgeFactor[2]);
        command.setLine(edgeFactor[3]);

        command.setShiftPressedState(e.isShiftDown());
        command.setSource(sourceDiagramElement);
        command.setTarget(targetDiagramElement);

        fireCommandPerformed(new CommandEvent(e.getSource(), e.getID(), "CreateNoteAnchor", command,
                e.getModifiers()));
        e.consume();
    }

    @Override
    protected LineConstraint getLineConstraint() {
        return NotationFactory.eINSTANCE.createLine();
    }

    @Override
    protected GNode getAppropriateNode(final GNode pickedNode) {
        final GNode nonLabelAncestorNode = getNonLabelAncestorNode(pickedNode);
        final DiagramElement diagramElement = getDiagramElement(nonLabelAncestorNode);
        if (isValid(diagramElement)) {
            return nonLabelAncestorNode;
        }
        return null;
    }

    private GNode getNonLabelAncestorNode(final GNode pickedNode) {
        final GNode ancestor = ViewElements.getAncestor(pickedNode, node -> {
            final DiagramElement diagramElement = getDiagramElement(node);
            return (diagramElement.eClass() != STAMPNotationPackage.Literals.STAMP_LABEL
                    && !(diagramElement instanceof Diagram));
        });
        return ancestor;
    }

    @Override
    protected boolean isValidTarget(final DiagramElement diagramElement) {
        return isValid(diagramElement);
    }

    @Override
    public boolean isLegalStart(final DiagramElement diagramElement) {
        return isValid(diagramElement);
    }

    @Override
    public boolean isLegalEnd(final DiagramElement diagramElement) {
        return isValid(diagramElement);
    }

    private boolean isValid(final DiagramElement diagramElement) {
        return ((diagramElement instanceof Shape && !(diagramElement instanceof GraphicalShape))
                || (diagramElement instanceof Edge && !hasModel(diagramElement, NoteAnchor.class)));
    }
}
