package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.dg.Image;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.ChangeImageCommand;
import net.astah.stpa.stamp.notation.STAMPGraphicalShape;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 共通図要素の画像を変更します。
 */
@Handler(targets = STAMPGraphicalShape.class)
@MenuItem(label = "%StampControlStructureDiagram_changeImage_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/image.png",
        tooltip = "%StampControlStructureDiagram_changeImage_tooltip",
        type = ItemKind.CHECK)
@MenuCommand(id = "stpa.ChangeImageCommand",
        category = "edit.change_image")
public class ChangeImageHandler {

    /**
     * 共通図要素の画像を変更するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final ChangeImageCommand command = new ChangeImageCommand();
        command.setUseTransaction(true);
        command.execute();
    }

    /**
     * 共通図要素の画像を変更するコマンドを実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 選択要素が1つ、かつ、画像の場合、true
        return Selections.size(selection) == 1
                && isImage(Selections.getFirstElement(selection));
    }

    private boolean isImage(final Object object) {
        return object instanceof STAMPGraphicalShape
                && ((STAMPGraphicalShape) object).getFigure() instanceof Image;
    }
}