package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jcontrol.CopyInProfessionalCommand;
import net.astah.jomt.jsystem.ViewGlobal;
import net.astah.jomt.jview.DiagramEditor;
import net.astah.stpa.stamp.notation.STAMPEdge;
import net.astah.stpa.stamp.notation.STAMPLabel;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.IsEnabled;
import net.astah.ui.menu.annotation.KeyBinding;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 選択した図要素をコピーします。
 */
@Handler
@MenuItem(label = "%StampControlStructureDiagram_copy_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/copy.png",
        tooltip = "%StampControlStructureDiagram_copy_tooltip",
        type = ItemKind.PUSH)
@MenuCommand(id = "stpa.CopyCommand",
        category = "edit.copy_paste")
@KeyBinding(keySequence = "control C")
public class CopyHandler {

    /**
     * 選択した図要素をコピーするコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            パラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final CopyInProfessionalCommand command = new CopyInProfessionalCommand();
        command.setUseTransaction(true);
        command.execute();
    }

    /**
     * 選択した図要素をコピーするコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // STAMPEdge、STAMPLabelのみの場合、false
        return Selections.stream(selection).anyMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        return !(object instanceof STAMPEdge || object instanceof STAMPLabel);
    }

    /**
     * 選択した図要素をコピーするメニューを活性にするかどうか確認します。
     * 
     * @return true メニューを活性にする / false 選択している要素がない場合、メニューを非活性にする
     */
    @IsEnabled
    public boolean isEnabled() {
        final DiagramEditor diagramEditor = ViewGlobal.viewManagerAccessor
                .getCurrentDiagramEditor();
        return diagramEditor != null && diagramEditor.getSelectedPresentations().length >= 1;
    }

}
