package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import net.astah.dd.di.DiagramElement;
import net.astah.dd.di.util.DiagramElements;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.stpa.stamp.ControlAction;
import net.astah.stpa.stamp.ControlLink;
import net.astah.stpa.stamp.notation.STAMPEdge;
import net.astah.stpa.stamp.notation.STAMPShape;
import net.astah.stpa.stamp.ui.control.CreateControlLoopDiagramCommand;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * リンク、または、コントロールアクションからコントロールループ図を作成します。
 */
@Handler(targets = { STAMPEdge.class, STAMPShape.class })
@MenuItem(label = "%StampControlStructureDiagram_createControlLoopDiagram_label",
        iconURI = "platform:/plugin/net.astah.stpa.stamp.notation.edit/icons/full/obj16/STAMPControlLoopDiagram.png",
        tooltip = "%StampControlStructureDiagram_createControlLoopDiagram_tooltip",
        type = ItemKind.PUSH)
@MenuCommand(id = "stpa.CreateControlLoopDiagramCommand",
        category = "edit.model")
public class CreateControlLoopDiagramHandler {

    /**
     * リンク、または、コントロールアクションからコントロールループ図を作成するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final CreateControlLoopDiagramCommand command = new CreateControlLoopDiagramCommand();
        final Object element = Selections.getFirstElement(selection);
        if (element instanceof DiagramElement) {
            command.setSelectedDiagramElement((DiagramElement) element);
            command.setUseTransaction(true);
            command.execute();
        }
    }

    /**
     * リンク、または、コントロールアクションからコントロールループ図を作成するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        final Object element = Selections.getFirstElement(selection);
        // 選択要素が1つ、かつ、リンクまたはコントロールアクションの場合、true
        return Selections.size(selection) == 1
                && (isControlLink(element) || isControlAction(element));
    }

    private boolean isControlLink(final Object object) {
        return object instanceof STAMPEdge
                && DiagramElements.isModelElementPresent((STAMPEdge) object, ControlLink.class);
    }

    private boolean isControlAction(final Object object) {
        return object instanceof STAMPShape
                && DiagramElements.isModelElementPresent((STAMPShape) object, ControlAction.class);
    }
}
