package net.astah.stpa.stamp.ui.menu.handler;

import static org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain.getEditingDomainFor;

import java.awt.event.ActionEvent;
import java.util.List;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.UnexecutableCommand;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.emf.edit.job.CommandJob;
import net.astah.golf.widget.IListSelection;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.ITableSelection;
import net.astah.jomt.jsystem.JomtGlobal;
import net.astah.jomt.jview.Editor;
import net.astah.jomt.jview.EditorPeer;
import net.astah.stpa.stamp.ui.view.swing.AccidentHazardSafetyConstraintTableEditorPeer;
import net.astah.table.Grid;
import net.astah.table.Tuple;
import net.astah.table.edit.command.DeleteTupleElementCommand;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * アクシデント表の安全制約を削除します。
 */
@Handler
@MenuItem(commands = @MenuCommand(id = "stpa.DeleteSafetyConstraintCommand",
        category = "edit.table"),
        label = "%StampTable_deleteSafetyConstraint_label",
        tooltip = "%StampTable_deleteSafetyConstraint_tooltip",
        type = ItemKind.PUSH)
public class DeleteSafetyConstraintHandler {

    private final int SAFETY_CONSTRAINT_ID_COLUMN = 4;

    private Command command = UnexecutableCommand.INSTANCE;

    /**
     * アクシデント表の安全制約を削除するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final Editor editor = JomtGlobal.viewManager.getCurrentEditor();
        final TransactionalEditingDomain domain = TransactionUtil
                .getEditingDomain(editor.getEditModel());
        final CommandJob job = new CommandJob(domain, command);
        job.schedule();
    }

    /**
     * アクシデント表の安全制約を削除するコマンドを実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        command.dispose();
        command = UnexecutableCommand.INSTANCE;
        if (selection instanceof ITableSelection) {
            command = getCommand((ITableSelection) selection);
        }
        return command.canExecute();
    }

    private Command getCommand(final ITableSelection selection) {
        final Object element = selection.getRowSelection().getFirstElement();
        if (element instanceof Tuple) {
            if (isSafetyConstraint(selection.getColumnSelection())) {
                final Tuple tuple = (Tuple) element;
                final AccidentHazardSafetyConstraintTableEditorPeer editorPeer = getEditorPeer();
                return createDeleteTupleElementCommand(tuple.getGrid(), tuple.getIndex(),
                        editorPeer.SAFETY_CONSTRAINT_INDEX);
            }
        }
        return command;
    }

    private boolean isSafetyConstraint(final IListSelection columnSelection) {
        final int[] indices = columnSelection.getIndices();
        if (indices.length > 0) {
            return indices[0] >= SAFETY_CONSTRAINT_ID_COLUMN;
        }
        return false;
    }

    private Command createDeleteTupleElementCommand(final Grid grid, final int rowIndex,
            final int elementIndex) {
        final EList<Tuple> rowTuples = grid.getRowTuples();
        if ((rowIndex >= 0) && (rowIndex < rowTuples.size())) {
            final Tuple rowTuple = rowTuples.get(rowIndex);
            return DeleteTupleElementCommand.create(getEditingDomainFor(grid), grid,
                    rowTuple, elementIndex);
        }
        return UnexecutableCommand.INSTANCE;
    }

    private AccidentHazardSafetyConstraintTableEditorPeer getEditorPeer() {
        final Editor editor = JomtGlobal.viewManager.getCurrentEditor();
        final EditorPeer editorPeer = editor.getEditorPeer();
        return (AccidentHazardSafetyConstraintTableEditorPeer) editorPeer;
    }

}
