package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.JMenuItem;

import net.astah.dd.dg.Polyline;
import net.astah.dd.dg.util.DGStyleResolver;
import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.ui.menu.annotation.AboutToShow;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 蛍光ペンの不透明度を指定します。
 * 
 * <p>
 * 0.1から0.9まで指定できます。
 */
@Handler(commands = "stpa.SetAlphaCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style"),
        path = "LineAlpha",
        label = "%StampControlStructureDiagram_lineAlpha_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_tooltip",
        type = ItemKind.MENU)
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style",
        parameters = "0.1"),
        path = "LineAlpha." + "0_1",
        label = "%StampControlStructureDiagram_lineAlpha_0_1_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_0_1_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style",
        parameters = "0.2"),
        path = "LineAlpha." + "0_2",
        label = "%StampControlStructureDiagram_lineAlpha_0_2_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_0_2_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style",
        parameters = "0.3"),
        path = "LineAlpha." + "0_3",
        label = "%StampControlStructureDiagram_lineAlpha_0_3_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_0_3_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style",
        parameters = "0.4"),
        path = "LineAlpha." + "0_4",
        label = "%StampControlStructureDiagram_lineAlpha_0_4_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_0_4_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style",
        parameters = "0.5"),
        path = "LineAlpha." + "0_5",
        label = "%StampControlStructureDiagram_lineAlpha_0_5_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_0_5_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style",
        parameters = "0.6"),
        path = "LineAlpha." + "0_6",
        label = "%StampControlStructureDiagram_lineAlpha_0_6_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_0_6_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style",
        parameters = "0.7"),
        path = "LineAlpha." + "0_7",
        label = "%StampControlStructureDiagram_lineAlpha_0_7_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_0_7_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style",
        parameters = "0.8"),
        path = "LineAlpha." + "0_8",
        label = "%StampControlStructureDiagram_lineAlpha_0_8_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_0_8_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetAlphaCommand",
        category = "edit.style",
        parameters = "0.9"),
        path = "LineAlpha." + "0_9",
        label = "%StampControlStructureDiagram_lineAlpha_0_9_label",
        tooltip = "%StampControlStructureDiagram_lineAlpha_0_9_tooltip",
        type = ItemKind.CHECK)
public class SetAlphaHandler {

    /**
     * 蛍光ペンの不透明度を指定するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメタ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction(
                "net.astah.jomt.jcontrol.SetAlpha%" + params.get(0));
        action.actionPerformed(e);
    }

    /**
     * 蛍光ペンの不透明度を指定するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 選択要素がHighlighterのみの場合、true
        return !Selections.isEmpty(selection)
                && Selections.stream(selection).allMatch(object -> isHighlighter(object));
    }

    private boolean isHighlighter(final Object object) {
        if (object instanceof Polyline) {
            final Polyline polyline = (Polyline) object;
            final double strokeOpacity = DGStyleResolver.getInstance(polyline)
                    .resolveStrokeOpacity(polyline);
            return strokeOpacity < 1;
        }
        return false;
    }

    /**
     * メニュー表示の直前に、蛍光ペンの不透明度に該当するメニューをチェックします。
     * 
     * @param selection
     *            選択要素
     * @param menuItem
     *            表示するメニューアイテム
     * @param params
     *            コマンドに指定されているパラメータ
     */
    @AboutToShow
    public void aboutToShow(final ISelection selection, final JMenuItem menuItem,
            final List<String> params) {
        if (params.isEmpty()) {
            return;
        }
        if (Selections.size(selection) > 1) {
            return;
        }
        final boolean isSameLineWidth = isSameLineWidth(Selections.getFirstElement(selection),
                params.get(0));
        menuItem.setSelected(isSameLineWidth);
    }

    private boolean isSameLineWidth(final Object object, final String alpha) {
        if (object instanceof Polyline) {
            final Polyline polyline = (Polyline) object;
            final double strokeOpacity = DGStyleResolver.getInstance(polyline)
                    .resolveStrokeOpacity(polyline);
            return strokeOpacity == Double.valueOf(alpha);
        }
        return false;
    }

}