package net.astah.stpa.stamp.ui.menu.handler;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.JMenuItem;

import net.astah.dd.dg.Group;
import net.astah.dd.dg.Rectangle;
import net.astah.dd.dg.Text;
import net.astah.golf.control.GolfAction;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.jmodel.StyleSupport;
import net.astah.ui.menu.annotation.AboutToShow;
import net.astah.ui.menu.annotation.CanExecute;
import net.astah.ui.menu.annotation.Execute;
import net.astah.ui.menu.annotation.Handler;
import net.astah.ui.menu.annotation.MenuCommand;
import net.astah.ui.menu.annotation.MenuItem;
import net.astah.ui.menu.annotation.MenuItem.ItemKind;

/**
 * 長方形の形状を設定します。
 * 
 * <p>
 * 長方形、角丸長方形。
 */
@Handler(commands = "stpa.SetRectTypeCommand")
@MenuItem(commands = @MenuCommand(id = "stpa.SetRectTypeCommand",
        category = "edit.rect"),
        path = "RectType",
        label = "%StampControlStructureDiagram_rectType_label",
        tooltip = "%StampControlStructureDiagram_rectType_tooltip",
        type = ItemKind.MENU)
@MenuItem(commands = @MenuCommand(id = "stpa.SetRectTypeCommand",
        category = "edit.rect",
        parameters = ""),
        path = "RectType.",
        label = "%StampControlStructureDiagram_rectType_rectangle_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/rect.png",
        tooltip = "%StampControlStructureDiagram_rectType_rectangle_tooltip",
        type = ItemKind.CHECK)
@MenuItem(commands = @MenuCommand(id = "stpa.SetRectTypeCommand",
        category = "edit.rect",
        parameters = StyleSupport.STYLE_RECT_TYPE_ROUND),
        path = "RectType." + StyleSupport.STYLE_RECT_TYPE_ROUND,
        label = "%StampControlStructureDiagram_rectType_round_label",
        iconURI = "platform:/plugin/net.astah.core.app/net/astah/jomt/resource/images/state.png",
        tooltip = "%StampControlStructureDiagram_rectType_round_tooltip",
        type = ItemKind.CHECK)
public class SetRectTypeHandler {

    /**
     * 長方形の形状を設定するコマンドを実行します。
     * 
     * @param selection
     *            選択要素
     * @param params
     *            コマンドのパラメータ
     * @param e
     *            アクションイベント
     */
    @Execute
    public void execute(final ISelection selection, final List<String> params,
            final ActionEvent e) {
        final GolfAction action = new GolfAction(
                "net.astah.jomt.jcontrol.SetRectType%" + params.get(0));
        action.actionPerformed(e);
    }

    /**
     * 長方形の形状を設定するコマンドが実行可能かどうか確認します。
     * 
     * @param selection
     *            選択要素
     * @return true コマンドが実行可能 / false コマンドが実行不可
     */
    @CanExecute
    public boolean canExecute(final ISelection selection) {
        // 選択要素が長方形、角丸長方形のみの場合、true
        return !Selections.isEmpty(selection)
                && Selections.stream(selection).allMatch(object -> isTarget(object));
    }

    private boolean isTarget(final Object object) {
        return object instanceof Rectangle
                && !isText((Rectangle) object);
    }

    private boolean isText(final Rectangle rect) {
        final Group group = rect.getGroup();
        if (group != null) {
            return group.getMembers().stream().anyMatch(Text.class::isInstance);
        }
        return false;
    }

    /**
     * メニュー表示の直前に、長方形の形状に合わせてメニューをチェックします。
     * 
     * @param selection
     *            選択要素
     * @param menuItem
     *            表示するメニューアイテム
     * @param params
     *            コマンドに指定されているパラメータ
     */
    @AboutToShow
    public void aboutToShow(final ISelection selection, final JMenuItem menuItem,
            final List<String> params) {
        if (params.isEmpty()) {
            return;
        }
        if (Selections.size(selection) > 1) {
            return;
        }
        final boolean sameRectType = isSameRectType(Selections.getFirstElement(selection),
                params.get(0));
        menuItem.setSelected(sameRectType);
    }

    private boolean isSameRectType(final Object object, final String rectType) {
        if (object instanceof Rectangle) {
            final double cornerRadius = ((Rectangle) object).getCornerRadius();
            switch (rectType) {
            case StyleSupport.STYLE_RECT_TYPE_ROUND:
                return cornerRadius > 0;
            case "": // Rectangle
                return cornerRadius == 0;
            }
        }
        return false;
    }
}