package net.astah.stpa.stamp.ui.table;

import static net.astah.stpa.stamp.STAMPPackage.Literals.LINK_END;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.ItemPropertyDescriptor;

import net.astah.emf.common.ecore.EObjects;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.stpa.stamp.Component;
import net.astah.stpa.stamp.ControlLink;
import net.astah.stpa.stamp.ControlStructure;
import net.astah.stpa.stamp.FeedbackLink;
import net.astah.stpa.stamp.Link;
import net.astah.stpa.stamp.LinkableElement;
import net.astah.stpa.stamp.Signal;
import net.astah.table.CellDescriptor;
import net.astah.table.ICellEvaluator;
import net.astah.table.Tuple;

/**
 * コンポーネント抽出表の入出力列を表示するための{@link IItemPropertyDescriptor} を定義したクラスです。
 */
public class InputOutputCellEvaluator implements ICellEvaluator {

    @Override
    public Object getElement(CellDescriptor cellDescriptor, Tuple tuple) {
        return null;
    }

    @Override
    public IItemPropertyDescriptor getHeaderPropertyDescriptor(CellDescriptor cellDescriptor,
            Tuple tuple) {
        return null;
    }

    @Override
    public IItemPropertyDescriptor getCellPropertyDescriptor(CellDescriptor cellDescriptor,
            Tuple rowTuple, Tuple columnTuple) {
        final EObject eObject = rowTuple.getElements().get(0).getObject();
        if (!(eObject instanceof Component)) {
            return null;
        }
        final Component component = (Component) eObject;
        final AdapterFactory adapterFactory = ItemProviders.getAdapterFactory(component);
        return new InputOutputPropertyDescriptor(adapterFactory, "", "", null, component);
    }

    @Override
    public Comparator<?> getComparator(CellDescriptor cellDescriptor) {
        return null;
    }

    /**
     * コンポーネント抽出表の入出力のプロパティ取得用{@link ItemPropertyDescriptor} です。
     */
    class InputOutputPropertyDescriptor extends ItemPropertyDescriptor {

        private final Component component;

        public InputOutputPropertyDescriptor(final AdapterFactory adapterFactory,
                final String displayName, final String description,
                final EReference[] parentReferences, final Component component) {
            super(adapterFactory, displayName, description, parentReferences);
            this.component = component;

        }

        @Override
        public Object getPropertyValue(final Object object) {
            final EList<Link> links = EObjects.getAncestor(ControlStructure.class, component)
                    .getLinks();
            final List<Signal> actions = new ArrayList<>();
            for (Link link : links) {
                final LinkableElement source = link.getSource();
                final LinkableElement target = link.getTarget();
                if (link instanceof ControlLink) {
                    ControlLink controlLink = (ControlLink) link;
                    if (isInput(source, target) || isOutput(source, target)) {
                        actions.addAll(controlLink.getActions());
                    }
                } else if (link instanceof FeedbackLink) {
                    FeedbackLink feedbackLink = (FeedbackLink) link;
                    if (isInput(source, target) || isOutput(source, target)) {
                        actions.addAll(feedbackLink.getFeedbacks());
                    }
                }
            }

            return actions;
        }

        private boolean isInput(final LinkableElement source, final LinkableElement target) {
            return LINK_END.isInstance(source) && target.equals(component);
        }

        private boolean isOutput(final LinkableElement source, final LinkableElement target) {
            return source.equals(component) && LINK_END.isInstance(target);
        }
    }
}
