package net.astah.stpa.stamp.ui.view.proptab;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.util.Collections;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.CreateChildCommand;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.emf.edit.command.DestroyCommand;
import net.astah.emf.edit.job.CommandJob;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.emf.edit.provider.swing.AdapterFactoryContentProvider;
import net.astah.emf.edit.provider.swing.JTableBuilder;
import net.astah.golf.view.swing.TableColumnProvider;
import net.astah.golf.widget.swing.ContentTableModel;
import net.astah.jomt.jview.proptab.PropertySourceTab;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.Signal;
import net.astah.stpa.stamp.ui.Messages;

/**
 * リンク、フィードバックリンクのコントロールアクション、フィードバック タブ汎用実装クラスです。
 */
public abstract class STAMPLinkSignalTab extends PropertySourceTab {

    private static final long serialVersionUID = 6699546200925915390L;

    private JTable table;
    private JScrollPane tablePanel;

    @Override
    protected void initPanel() {
        super.initPanel();
        if (baseItemsPanel == null) {
            return;
        }

        // テーブル
        table = createTable();
        if (tablePanel == null) {
            tablePanel = new JScrollPane();
        }
        tablePanel.setViewportView(table);

        // ボタン
        JButton addButton = createAddButton();
        JButton delButton = createDeleteButton();

        // ボタンパネル
        JPanel buttonPanel = new JPanel(new GridLayout(1, 0));
        buttonPanel.add(addButton);
        buttonPanel.add(delButton);

        add(tablePanel, BorderLayout.CENTER);
        add(buttonPanel, BorderLayout.SOUTH);
    }

    private JButton createDeleteButton() {
        JButton delButton = new JButton(Messages.projectview_delete_button_label);
        delButton.addActionListener(getDeleteButtonActionListener());
        return delButton;
    }

    /**
     * 追加ボタンを作成し、返します。
     * 
     * @return タブ内に表示される追加ボタン
     */
    protected JButton createAddButton() {
        JButton addButton = new JButton(Messages.projectview_add_button_label);
        addButton.addActionListener(getAddButtonActionListener());
        return addButton;
    }

    private JTable createTable() {
        EditingDomain domain = AdapterFactoryEditingDomain.getEditingDomainFor(getModel());
        AdapterFactory adapterFactory = ItemProviders.getAdapterFactory(domain);
        AdapterFactoryContentProvider contentProvider = createContentProvider(domain,
                adapterFactory);
        table = JTableBuilder.builder(domain).adapterFactory(adapterFactory)
                .contentProvider(contentProvider).input(getInput())
                .addColumn(TableColumnProvider.idProperty(Messages.precondition_table_name_label,
                        STAMPPackage.Literals.NAMED_ELEMENT__NAME.getName()))
                .build();
        table.setDragEnabled(false);
        table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        return table;
    }

    /**
     * テーブルに表示するオブジェクトを供給する {@link AdapterFactoryContentProvider} を構築します。
     * 
     * @param domain
     *            EditingDomain
     * @param adapterFactory
     *            AdapterFactory
     * @return 構築した {@code AdapterFactoryContentProvider}
     */
    protected abstract AdapterFactoryContentProvider createContentProvider(EditingDomain domain,
            AdapterFactory adapterFactory);

    /**
     * コントロールアクション、フィードバック作成を行う抽象クラスです。
     * 
     * @return 作成したコントロールアクション、フィードバック
     */
    protected abstract Signal createSignalAction();

    private ActionListener getAddButtonActionListener() {
        return actionEvent -> {
            TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(model);

            Signal signalAction = createSignalAction();
            Command cmd = CreateChildCommand.create(domain, model,
                    new CommandParameter(model, STAMPPackage.Literals.LINK__SIGNAL, signalAction),
                    Collections.singleton(model));
            CommandJob job = new CommandJob(domain, cmd);
            job.schedule();
        };
    }

    private ActionListener getDeleteButtonActionListener() {
        return actionEvent -> {
            int selectedRow = table.getSelectedRow();
            if (selectedRow >= 0) {
                // 編集状態解除
                if (table.isEditing()) {
                    table.getCellEditor().stopCellEditing();
                }

                // コマンド作成
                Object selectedObject = ((ContentTableModel) table.getModel())
                        .getRowAt(selectedRow);
                TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(model);
                Command command = DestroyCommand.create(domain, selectedObject);

                // コマンド実行
                CommandJob job = new CommandJob(domain, command);
                job.schedule();
            }
        };
    }

    @Override
    public void setInitialized(boolean b) {
        super.setInitialized(b);
        if (!b) {
            if (table != null) {
                JTableBuilder.dispose(table);
                tablePanel.remove(table);
                table = null;
            }
        }
    }

}
