package net.astah.stpa.stamp.ui.view.swing;

import static java.util.Arrays.asList;
import static java.util.Collections.singleton;
import static net.astah.stpa.stamp.STAMPPackage.Literals.COMPONENT;
import static net.astah.stpa.stamp.STAMPPackage.Literals.COMPONENT__IS_ANALYSIS_TARGET;
import static net.astah.stpa.stamp.STAMPPackage.Literals.COMPONENT__REMARKS;
import static net.astah.stpa.stamp.STAMPPackage.Literals.COMPONENT__RESPONSIBILITY;
import static net.astah.stpa.stamp.STAMPPackage.Literals.CONTROL_STRUCTURE__COMPONENT;
import static net.astah.stpa.stamp.STAMPPackage.Literals.NAMED_ELEMENT__NAME;
import static org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain.getEditingDomainFor;

import java.awt.Container;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.common.command.UnexecutableCommand;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.CreateChildCommand;
import org.eclipse.emf.edit.command.MoveCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.IItemProviderDecorator;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.util.TransactionUtil;

import com.google.common.base.Strings;

import net.astah.dd.di.Diagram;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.emf.edit.provider.swing.JTableBuilder;
import net.astah.emf.edit.provider.swing.PropertyEditorFactory;
import net.astah.emf.edit.provider.swing.PropertyEditorFactory.Registry;
import net.astah.emf.edit.provider.swing.PropertyEditorFactoryRegistryImpl;
import net.astah.emf.edit.provider.swing.editor.CheckBoxCellEditor;
import net.astah.golf.control.CommandManager;
import net.astah.golf.geom2D.Pnt2d;
import net.astah.golf.model.EntityStoreEvent;
import net.astah.golf.view.swing.PropertyCellEditor;
import net.astah.golf.widget.CJComponent;
import net.astah.golf.widget.IListSelection;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.ITableSelection;
import net.astah.golf.widget.TableSelection;
import net.astah.golf.widget.swing.GenericTable;
import net.astah.golf.widget.swing.Selections;
import net.astah.jomt.jview.MessageManager;
import net.astah.notation.TableView;
import net.astah.stpa.stamp.Component;
import net.astah.stpa.stamp.ControlStructure;
import net.astah.stpa.stamp.LinkableElement;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.STAMPPackage;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.notation.ComponentExtractingTable;
import net.astah.stpa.stamp.notation.STAMPControlStructureDiagram;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.control.ModifyAnalysisTargetCommand;
import net.astah.stpa.stamp.ui.control.ModifyControlStructureDiagramElementsCommand;
import net.astah.stpa.stamp.ui.menu.handler.AddComponentHandler;
import net.astah.stpa.stamp.ui.menu.handler.CreateControlStructureDiagramHandler;
import net.astah.stpa.stamp.ui.menu.handler.DeleteComponentHandler;
import net.astah.stpa.stamp.ui.table.ComponentExtractingTableColumnProvider;
import net.astah.stpa.stamp.ui.table.ControlActionCellEvaluator;
import net.astah.stpa.stamp.ui.table.FeedbackCellEvaluator;
import net.astah.stpa.stamp.ui.table.InputOutputCellEvaluator;
import net.astah.table.DynamicDataset;
import net.astah.table.ElementSet;
import net.astah.table.FixedDataset;
import net.astah.table.Grid;
import net.astah.table.NativeCellDescriptor;
import net.astah.table.ObjectDataQuery;
import net.astah.table.ObjectElement;
import net.astah.table.PropertyCellDescriptor;
import net.astah.table.Table;
import net.astah.table.TableFactory;
import net.astah.table.Tuple;
import net.astah.table.edit.provider.GenericTableAdapterFactory;
import net.astah.table.edit.provider.GridItemProviderDecorator;
import net.astah.table.ui.swing.GenericTableColumnProvider;
import net.astah.table.ui.swing.GenericTableTableTransferHandler;
import net.astah.ui.menu.annotation.Context;

/**
 * コンポーネント抽出表のTableEditorPeer 実装です。
 */
@Context(id = "net.astah.stpa.stamp.notation.ComponentExtractingTable",
        classes = { // classesの記述順でメニューを表示する
                AddComponentHandler.class,
                DeleteComponentHandler.class,
                CreateControlStructureDiagramHandler.class
        })
public class ComponentExtractingTableEditorPeer extends STPATableEditorPeer {
    public static final int COMPONENT_INDEX = 0;
    public static final String ANNOTATION__COMPONENT_EXTRACTING_TABLE = "componentExtractingTable";

    private static final double COMPONENT_START_X = 0;
    private static final double COMPONENT_START_Y = 10;

    private JLabel relationDiagramName;

    @Override
    public void initialize(final Diagram diagram, final CJComponent parent) {
        super.initialize(diagram, parent);
        final JTable table = (JTable) getTable().getComponent();
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setCellSelectionEnabled(true);
    }

    @Override
    public void update(final EntityStoreEvent e) {
        super.update(e);
        updateInfo();
    }

    private void updateInfo() {
        final ComponentExtractingTable table = (ComponentExtractingTable) getDiagram();
        final STAMPControlStructureDiagram csDiagram = table.getControlStructureDiagram();
        relationDiagramName
                .setText(csDiagram != null ? Strings.emptyToNull(csDiagram.getName()) : "");
    }

    @Override
    protected float[] getColumnsWidthRatio() {
        return new float[] { 1 / 23f, 3 / 23f, 5 / 23f, 3 / 23f, 3 / 23f, 3 / 23f, 5 / 23f };
    }

    @Override
    protected Table prepareTableModel(final TableView view) {
        final STPAAnalysis analysis = ((ComponentExtractingTable) getDiagram()).getScope();
        final ObjectDataQuery row0 = TableFactory.eINSTANCE.createObjectDataQuery();
        row0.getObjects().add(analysis);
        row0.setMetaclass(COMPONENT);
        row0.setTraversal(true);

        final DynamicDataset row = TableFactory.eINSTANCE.createDynamicDataset();
        row.getQueries().addAll(asList(row0));

        final ObjectElement columnTarget = TableFactory.eINSTANCE.createObjectElement();
        columnTarget.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(
                        Messages.component_extracting_table_select_target_label));
        final PropertyCellDescriptor column0Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column0Cell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(column0Cell, EFeaturePath.build(COMPONENT__IS_ANALYSIS_TARGET));
        columnTarget.setCell(column0Cell);

        final ObjectElement columnComponent = TableFactory.eINSTANCE.createObjectElement();
        columnComponent.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.component_extracting_table_component_label));
        final PropertyCellDescriptor column1Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column1Cell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(column1Cell, EFeaturePath.build(NAMED_ELEMENT__NAME));
        column1Cell.setLineWrap(true);
        columnComponent.setCell(column1Cell);

        final ObjectElement columnResponsibility = TableFactory.eINSTANCE.createObjectElement();
        columnResponsibility.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(
                        Messages.component_extracting_table_responsibility_label));
        final PropertyCellDescriptor column2Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column2Cell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(column2Cell, EFeaturePath.build(COMPONENT__RESPONSIBILITY));
        column2Cell.setLineWrap(true);
        columnResponsibility.setCell(column2Cell);

        final ObjectElement columnControlAction = TableFactory.eINSTANCE.createObjectElement();
        columnControlAction.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.component_extracting_table_ca_label));
        final NativeCellDescriptor column3Cell = TableFactory.eINSTANCE
                .createNativeCellDescriptor();
        column3Cell.setImplementationClass(ControlActionCellEvaluator.class);
        column3Cell.setLineWrap(true);
        columnControlAction.setCell(column3Cell);

        final ObjectElement columnFeedback = TableFactory.eINSTANCE.createObjectElement();
        columnFeedback.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.component_extracting_table_fb_label));
        final NativeCellDescriptor column4Cell = TableFactory.eINSTANCE
                .createNativeCellDescriptor();
        column4Cell.setImplementationClass(FeedbackCellEvaluator.class);
        column4Cell.setLineWrap(true);
        columnFeedback.setCell(column4Cell);

        final ObjectElement columnInputOutput = TableFactory.eINSTANCE.createObjectElement();
        columnInputOutput.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.component_extracting_table_input_output_label));
        final NativeCellDescriptor column5Cell = TableFactory.eINSTANCE
                .createNativeCellDescriptor();
        column5Cell.setImplementationClass(InputOutputCellEvaluator.class);
        column5Cell.setLineWrap(true);
        columnInputOutput.setCell(column5Cell);

        final ObjectElement columnRemarks = TableFactory.eINSTANCE.createObjectElement();
        columnRemarks.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.component_extracting_table_remarks_label));
        final PropertyCellDescriptor column6Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column6Cell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(column6Cell, EFeaturePath.build(COMPONENT__REMARKS));
        column6Cell.setLineWrap(true);
        columnRemarks.setCell(column6Cell);

        final ElementSet columnSet = TableFactory.eINSTANCE.createElementSet();
        columnSet.getElements().addAll(asList(columnTarget, columnComponent, columnResponsibility,
                columnControlAction, columnFeedback, columnInputOutput, columnRemarks));
        final FixedDataset column = TableFactory.eINSTANCE.createFixedDataset();
        column.getElementSets().add(columnSet);

        final Table table = TableFactory.eINSTANCE.createTable();
        table.getColumns().add(column);
        table.getRows().add(row);
        return table;
    }

    @Override
    protected JTable createJTable(final AdapterFactory adapterFactory, final Grid grid) {
        final GenericTableColumnProvider columnProvider = new ComponentExtractingTableColumnProvider();

        final PropertyEditorFactory.Registry registry = new PropertyEditorFactoryRegistryImpl(
                Registry.INSTANCE);
        final PropertyEditorFactory factory = new CheckBoxCellEditor.DescriptorImpl() {
            @Override
            public PropertyCellEditor createPropertyEditor(Object object,
                    IItemPropertyDescriptor propertyDescriptor) {
                final JCheckBox checkBox = new JCheckBox();
                final CheckBoxCellEditor cellEditor = new CheckBoxCellEditor(checkBox) {
                    private static final long serialVersionUID = 1L;

                    @Override
                    public boolean stopCellEditing() {
                        return stopEditing();
                    }

                    private synchronized boolean stopEditing() {

                        boolean returnValue = super.stopCellEditing();
                        ControlStructure controlStructure = getScope().getControlStructures()
                                .stream()
                                .findFirst().orElse(null);
                        if (controlStructure == null) {
                            return returnValue;
                        }

                        ModifyAnalysisTargetCommand cmd = new ModifyAnalysisTargetCommand();
                        cmd.setControlStructure(controlStructure);
                        CommandManager.instance().callCommand(cmd);

                        return returnValue;
                    }
                };

                checkBox.addItemListener(e -> cellEditor.stopCellEditing());
                return cellEditor;
            }
        };
        registry.getFeatureMap().put(COMPONENT__IS_ANALYSIS_TARGET, factory);

        columnProvider.setAdapterFactory(adapterFactory);
        columnProvider.setGrid(grid);
        final TransactionalEditingDomain txDomain = TransactionUtil.getEditingDomain(getDiagram());
        return JTableBuilder.builder(txDomain)
                .adapterFactory(adapterFactory)
                .columnProvider(columnProvider)
                .transferHandler(new GenericTableTableTransferHandler(txDomain, domain))
                .input(grid)
                .propertyEditorFactoryRegistry(registry)
                .font(resolveTableFont())
                .reorderingAllowed(false)
                .build(new GenericTable());
    }

    /**
     * コンポーネント抽出表のスコープを取得します
     * 
     * @return コンポーネント抽出表のスコープ
     */
    public STPAAnalysis getScope() {
        return ((ComponentExtractingTable) getDiagram()).getScope();
    }

    @Override
    protected AdapterFactory createTableAdapterFactory() {
        final STPAAnalysis scope = getScope();
        final TransactionalEditingDomain domain = TransactionUtil.getEditingDomain(getDiagram());
        return new GenericTableAdapterFactory(ItemProviders.getAdapterFactory(domain)) {

            @Override
            protected IItemProviderDecorator createGridDecorator() {
                return new GridItemProviderDecorator(this) {

                    @Override
                    public Command createCommand(Object object, EditingDomain domain,
                            Class<? extends Command> commandClass,
                            CommandParameter commandParameter) {
                        if ((commandClass == MoveCommand.class)) {
                            final Tuple targetTuple = (Tuple) Selections
                                    .getFirstElement(getTable().getSelection());
                            final Component target = (Component) targetTuple.getElementObject(0);
                            final ControlStructure targetOwner = target.getOwner();
                            if (targetOwner == null) {
                                // ネストコンポーネントなら並べ替えできない
                                MessageManager.displayErrorMessageDialog(
                                        Messages.nested_component_cannot_be_sorted);
                                return UnexecutableCommand.INSTANCE;
                            }
                        }

                        return super.createCommand(object, domain, commandClass, commandParameter);
                    }

                    @Override
                    protected Command createPrepareRowTupleCommand(final Grid grid,
                            final Tuple tuple,
                            final EditingDomain domain) {
                        return prepareComponentCommand(tuple);
                    }

                    @Override
                    protected Command createMoveRowElementCommand(final EObject owner,
                            final Tuple tuple,
                            final int index,
                            final EditingDomain defaultDomain) {
                        return createMoveComponentExtractingCommand(owner, tuple, index, domain);
                    }
                };
            }

            protected Command createMoveComponentExtractingCommand(final EObject owner,
                    final Tuple tuple,
                    final int index,
                    final EditingDomain defaultDomain) {
                final Tuple targetTuple = grid.getRowTuples().get(index);
                final Component target = (Component) targetTuple.getElementObject(0);
                final ControlStructure targetOwner = target.getOwner();
                if (targetOwner == null) {
                    return UnexecutableCommand.INSTANCE;
                }
                final int targetIndex = targetOwner.getComponents().indexOf(target);

                final EObject value = tuple.getElementObject(0);
                return MoveCommand.create(getEditingDomainFor(targetOwner), targetOwner,
                        CONTROL_STRUCTURE__COMPONENT, value, targetIndex);
            }

            protected Command prepareComponentCommand(final Tuple rowTuple) {
                CompoundCommand command = new CompoundCommand();

                final Component component = STAMPFactory.eINSTANCE.createComponent();
                final EList<ControlStructure> controlStructures = scope.getControlStructures();
                final ControlStructure controlStructure;

                if (controlStructures.isEmpty()) {
                    controlStructure = STAMPFactory.eINSTANCE.createControlStructure();
                    final CommandParameter parameter = new CommandParameter(scope,
                            STAMPPackage.Literals.STPA_ANALYSIS__CONTROL_STRUCTURE,
                            controlStructure, 0);
                    command.append(
                            CreateChildCommand.create(domain, scope, parameter, singleton(scope)));
                } else {
                    controlStructure = controlStructures.get(0);
                }

                final EList<Component> components = controlStructure.getComponents();
                final int index = components != null ? components.size() : 0;
                final CommandParameter parameter = new CommandParameter(controlStructure,
                        STAMPPackage.Literals.CONTROL_STRUCTURE__COMPONENT, component, index);

                command.append(
                        CreateChildCommand.create(domain, controlStructure, parameter,
                                singleton(controlStructure)));
                command.append(
                        SetCommand.create(domain, component, COMPONENT__IS_ANALYSIS_TARGET, true));

                if (existRelationDiagram()) {
                    final EList<LinkableElement> elements = new BasicEList<>();
                    if (components != null) {
                        elements.addAll(components);
                    }
                    elements.add(component);
                    command.append(
                            new ModifyControlStructureDiagramElementsCommand(domain, elements,
                                    controlStructure, getDefaultPoint()));
                }
                return command;
            }

            private boolean existRelationDiagram() {
                final ComponentExtractingTable table = (ComponentExtractingTable) getDiagram();
                return table.getControlStructureDiagram() != null;
            }

            private Pnt2d getDefaultPoint() {
                return new Pnt2d(COMPONENT_START_X, COMPONENT_START_Y);
            }
        };
    }

    @Override
    protected Container createNorthPanel() {
        final Container panel = super.createNorthPanel();
        panel.add(createInfoPanel(), 0);
        updateInfo();
        return panel;
    }

    private Container createInfoPanel() {
        final JPanel infoPanel = new JPanel();
        infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));
        infoPanel.setBorder(
                new CompoundBorder(infoPanel.getBorder(), new EmptyBorder(10, 10, 0, 0)));

        // 同期するCS図の情報
        final JPanel relationDiagramPanel = new JPanel();
        relationDiagramPanel.setLayout(new BoxLayout(relationDiagramPanel, BoxLayout.X_AXIS));
        relationDiagramName = new JLabel();
        relationDiagramPanel
                .add(new JLabel(Messages.component_extracting_table_relation_diagram_label + " ："));
        relationDiagramPanel.add(Box.createHorizontalStrut(10));
        relationDiagramPanel.add(relationDiagramName);
        relationDiagramPanel.add(Box.createHorizontalGlue());

        infoPanel.add(relationDiagramPanel);
        return infoPanel;
    }

    @Override
    protected ISelection getEditorSelection(final ITableSelection selection) {
        final IListSelection columnSelection = selection.getColumnSelection();
        final int[] columnIndices = columnSelection.getIndices();
        if (columnIndices.length > 0) {
            final List<Object> elements = new ArrayList<>(selection.size());
            for (final Tuple tuple : Selections.iterable(selection, Tuple.class)) {
                elements.add(tuple.getElementObject(0));
            }
            final IListSelection rowSelection = selection.getRowSelection();
            return new TableSelection(elements, rowSelection, columnSelection);
        }
        return super.getEditorSelection(selection);
    }

}
