package net.astah.stpa.stamp.ui.view.swing;

import static java.util.Arrays.asList;
import static java.util.Collections.singleton;
import static net.astah.stpa.stamp.STAMPPackage.Literals.IDENTIFIED_ELEMENT__DESCRIPTION;
import static net.astah.stpa.stamp.STAMPPackage.Literals.IDENTIFIED_ELEMENT__ID;
import static net.astah.stpa.stamp.STAMPPackage.Literals.PRECONDITION;
import static net.astah.stpa.stamp.STAMPPackage.Literals.STPA_ANALYSIS__PRECONDITION;
import static org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain.getEditingDomainFor;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.command.CreateChildCommand;
import org.eclipse.emf.edit.command.MoveCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.provider.IItemProviderDecorator;
import org.eclipse.emf.transaction.util.TransactionUtil;

import net.astah.dd.di.Diagram;
import net.astah.emf.common.notify.EFeaturePath;
import net.astah.emf.edit.provider.ItemProviders;
import net.astah.golf.widget.CJComponent;
import net.astah.golf.widget.IListSelection;
import net.astah.golf.widget.ISelection;
import net.astah.golf.widget.ITableSelection;
import net.astah.golf.widget.TableSelection;
import net.astah.golf.widget.swing.Selections;
import net.astah.notation.TableView;
import net.astah.stpa.stamp.Precondition;
import net.astah.stpa.stamp.STAMPFactory;
import net.astah.stpa.stamp.STPAAnalysis;
import net.astah.stpa.stamp.notation.PreconditionTable;
import net.astah.stpa.stamp.ui.Messages;
import net.astah.stpa.stamp.ui.menu.handler.AddPreconditionHandler;
import net.astah.stpa.stamp.ui.menu.handler.DeletePreconditionHandler;
import net.astah.table.DynamicDataset;
import net.astah.table.Element;
import net.astah.table.ElementSet;
import net.astah.table.FixedDataset;
import net.astah.table.Grid;
import net.astah.table.ObjectDataQuery;
import net.astah.table.ObjectElement;
import net.astah.table.PropertyCellDescriptor;
import net.astah.table.Table;
import net.astah.table.TableFactory;
import net.astah.table.Tuple;
import net.astah.table.edit.provider.GenericTableAdapterFactory;
import net.astah.table.edit.provider.GridItemProviderDecorator;
import net.astah.ui.menu.annotation.Context;

/**
 * 前提条件表のTableEditorPeer 実装です。
 */
@Context(id = "net.astah.stpa.stamp.notation.PreconditionTable",
        classes = { // classesの記述順でメニューを表示する
                AddPreconditionHandler.class,
                DeletePreconditionHandler.class
        })
public class PreconditionTableEditorPeer extends STPATableEditorPeer {

    @Override
    public void initialize(final Diagram diagram, final CJComponent parent) {
        super.initialize(diagram, parent);
        final JTable table = (JTable) getTable().getComponent();
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setCellSelectionEnabled(true);
    }

    @Override
    protected float[] getColumnsWidthRatio() {
        return new float[] { 1 / 10f, 9 / 10f };
    }

    @Override
    protected Table prepareTableModel(final TableView model) {
        final STPAAnalysis analysis = ((PreconditionTable) model).getScope();

        final ObjectDataQuery row0 = TableFactory.eINSTANCE.createObjectDataQuery();
        row0.getObjects().add(analysis);
        row0.setMetaclass(PRECONDITION);
        row0.setDistinguishable(true);
        row0.setTraversal(true);

        final DynamicDataset row = TableFactory.eINSTANCE.createDynamicDataset();
        row.getQueries().addAll(asList(row0));

        final ObjectElement columnId = TableFactory.eINSTANCE.createObjectElement();
        columnId.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.precondition_table_id_label));
        final PropertyCellDescriptor column0Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column0Cell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(column0Cell, EFeaturePath.build(IDENTIFIED_ELEMENT__ID));
        column0Cell.setLineWrap(true);
        columnId.setCell(column0Cell);

        final ObjectElement columnName = TableFactory.eINSTANCE.createObjectElement();
        columnName.setHeader(TableFactory.eINSTANCE
                .createFixedCellDescriptor(Messages.precondition_table_name_label));
        final PropertyCellDescriptor column1Cell = TableFactory.eINSTANCE
                .createPropertyCellDescriptor();
        column1Cell.setOrigin(row.getQueries().indexOf(row0));
        applyFeaturePath(column1Cell, EFeaturePath.build(IDENTIFIED_ELEMENT__DESCRIPTION));
        column1Cell.setLineWrap(true);
        columnName.setCell(column1Cell);

        final ElementSet columnSet = TableFactory.eINSTANCE.createElementSet();
        columnSet.getElements().addAll(asList(columnId, columnName));
        final FixedDataset column = TableFactory.eINSTANCE.createFixedDataset();
        column.getElementSets().add(columnSet);

        final Table table = TableFactory.eINSTANCE.createTable();
        table.getColumns().add(column);
        table.getRows().add(row);
        return table;
    }

    @Override
    protected AdapterFactory createTableAdapterFactory() {
        final EditingDomain domain = TransactionUtil.getEditingDomain(getDiagram());
        final STPAAnalysis scope = ((PreconditionTable) getDiagram()).getScope();
        return new GenericTableAdapterFactory(ItemProviders.getAdapterFactory(domain)) {

            @Override
            protected IItemProviderDecorator createGridDecorator() {
                return new GridItemProviderDecorator(this) {
                    @Override
                    protected Command createPrepareRowTupleCommand(final Grid grid,
                            final Tuple tuple,
                            final EditingDomain domain) {
                        return preparePreconditionCommand(tuple);
                    }

                    @Override
                    protected Command createMoveRowElementCommand(final EObject owner,
                            final Tuple tuple,
                            final int index,
                            final EditingDomain defaultDomain) {
                        return createMovePreconditionCommand(owner, tuple, index, defaultDomain);
                    }
                };
            }

            protected Command preparePreconditionCommand(final Tuple rowTuple) {
                final Precondition precondition = STAMPFactory.eINSTANCE.createPrecondition();
                final EList<Element> elements = rowTuple.getElements();
                elements.get(0).setObject(precondition);

                final EditingDomain domain = getEditingDomainFor(scope);
                final CommandParameter childDescription = new CommandParameter(scope,
                        STPA_ANALYSIS__PRECONDITION, precondition);
                return CreateChildCommand.create(domain, scope, childDescription, singleton(scope));
            }

            protected Command createMovePreconditionCommand(final EObject owner,
                    final Tuple tuple,
                    final int index,
                    final EditingDomain defaultDomain) {
                final Tuple targetTuple = grid.getRowTuples().get(index);
                final Precondition target = (Precondition) targetTuple.getElementObject(0);
                final STPAAnalysis targetOwner = target.getOwner();
                final int targetIndex = scope.getPreconditions().indexOf(target);

                final EObject value = tuple.getElementObject(0);
                return MoveCommand.create(getEditingDomainFor(targetOwner), targetOwner,
                        STPA_ANALYSIS__PRECONDITION, value, targetIndex);
            }

        };
    }

    @Override
    protected ISelection getEditorSelection(final ITableSelection selection) {
        final IListSelection columnSelection = selection.getColumnSelection();
        final int[] columnIndices = columnSelection.getIndices();
        if (columnIndices.length > 0) {
            final List<Object> elements = new ArrayList<>(selection.size());
            for (final Tuple tuple : Selections.iterable(selection, Tuple.class)) {
                elements.add(tuple.getElementObject(0));
            }
            final IListSelection rowSelection = selection.getRowSelection();
            return new TableSelection(elements, rowSelection, columnSelection);
        }
        return super.getEditorSelection(selection);
    }

}
