package net.astah.stpa.stamp.constraints;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.validation.IValidationContext;
import org.eclipse.emf.validation.model.ConstraintSeverity;

import com.google.common.base.Strings;

import net.astah.emf.validation.java.ModeledJavaConstraintProvider;
import net.astah.emf.validation.java.annotation.Constraint;
import net.astah.emf.validation.java.annotation.ConstraintTarget;
import net.astah.emf.validation.java.annotation.EvaluationModeType;
import net.astah.jomt.jsystem.PropertyManager;
import net.astah.stpa.stamp.NamedElement;
import net.astah.stpa.stamp.STAMPPackage;

/**
 * STAMP/STPA のモデル妥当性制約の検証ルールを実装します。
 */
@Constraint(evaluationMode = EvaluationModeType.LIVE, severity = ConstraintSeverity.CANCEL)
public class STAMPConstraintProvider extends ModeledJavaConstraintProvider {

    /**
     * {@link NamedElement} に名前が設定されていることを確認する制約ルールです。
     * 
     * @param ctx
     *            {@code IValidationContext}
     * @param target
     *            検証対象のオブジェクト
     * @return 検証結果のオブジェクト
     */
    @Constraint(message = "%model_element_no_name_error.message",
            targets = {
                    @ConstraintTarget(type = NamedElement.class,
                            events = { Notification.SET, Notification.UNSET },
                            eFeatures = STAMPPackage.NAMED_ELEMENT__NAME) })
    public IStatus validateHasName(final IValidationContext ctx, final NamedElement target) {
        String name = target.getName();
        if (Strings.isNullOrEmpty(name)) {
            return ctx.createFailureStatus(target);
        }
        return ctx.createSuccessStatus();
    }

    /**
     * 多言語対応された検証結果のメッセージを取得します。
     * 
     * @param key
     *            取得するメッセージのキー
     * @return 多言語対応された検証結果のメッセージ
     */
    @Override
    protected String getString(String key) {
        return PropertyManager.getUmlMessageResourceBundle().getString(key);
    }
}
